@@ -27,17 +27,16 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "waylandlocker.h"
 #include "logind.h"
 #include "kscreensaversettings.h"
+#include "powermanagement_inhibition.h"
 #include <config-kscreenlocker.h>
 #include <config-X11.h>
 #include "waylandserver.h"
 // KDE
-#include <KActionCollection>
 #include <KAuthorized>
 #include <KIdleTime>
 #include <KLocalizedString>
 #include <KNotification>
 #include <KGlobalAccel>
-#include <KCrash>
 
 //kwayland
 #include <KWayland/Server/display.h>
@@ -45,6 +44,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 // Qt
 #include <QAction>
+#include <QKeyEvent>
 #include <QTimer>
 #include <QProcess>
 #include <QX11Info>
@@ -79,11 +79,11 @@ KSldApp* KSldApp::self()
 
 KSldApp::KSldApp(QObject * parent)
     : QObject(parent)
-    , m_actionCollection(NULL)
     , m_lockState(Unlocked)
     , m_lockProcess(NULL)
     , m_lockWindow(NULL)
     , m_waylandServer(new WaylandServer(this))
+    , m_waylandDisplay(nullptr)
     , m_lockedTimer(QElapsedTimer())
     , m_idleId(0)
     , m_lockGrace(0)
@@ -91,6 +91,8 @@ KSldApp::KSldApp(QObject * parent)
     , m_graceTimer(new QTimer(this))
     , m_inhibitCounter(0)
     , m_logind(nullptr)
+    , m_greeterEnv(QProcessEnvironment::systemEnvironment())
+    , m_powerManagementInhibition(new PowerManagementInhibition(this))
 {
     m_isX11 = QX11Info::isPlatformX11();
     m_isWayland = QGuiApplication::platformName().startsWith( QLatin1String("wayland"), Qt::CaseInsensitive);
@@ -110,7 +112,6 @@ void KSldApp::cleanUp()
     if (m_lockProcess && m_lockProcess->state() != QProcess::NotRunning) {
         m_lockProcess->terminate();
     }
-    delete m_actionCollection;
     delete m_lockProcess;
     delete m_lockWindow;
 
@@ -161,17 +162,16 @@ void KSldApp::initializeX11()
 
 void KSldApp::initialize()
 {
-    KCrash::setFlags(KCrash::AutoRestart);
     if (m_isX11) {
         initializeX11();
     }
 
     // Global keys
-    m_actionCollection = new KActionCollection(this);
-
     if (KAuthorized::authorizeKAction(QStringLiteral("lock_screen"))) {
         qDebug() << "Configuring Lock Action";
-        QAction *a = m_actionCollection->addAction(QStringLiteral("Lock Session"));
+        QAction *a = new QAction(this);
+        a->setObjectName(QStringLiteral("Lock Session"));
+        a->setProperty("componentName", QStringLiteral("ksmserver"));
         a->setText(i18n("Lock Session"));
         KGlobalAccel::self()->setGlobalShortcut(a, QList<QKeySequence>() << Qt::ALT+Qt::CTRL+Qt::Key_L << Qt::Key_ScreenSaver );
         connect(a, &QAction::triggered, this,
@@ -180,7 +180,6 @@ void KSldApp::initialize()
             }
         );
     }
-    m_actionCollection->readSettings();
 
     // idle support
     auto idleTimeSignal = static_cast<void (KIdleTime:: *)(int)>(&KIdleTime::timeoutReached);
@@ -209,6 +208,7 @@ void KSldApp::initialize()
     );
 
     m_lockProcess = new QProcess();
+    m_lockProcess->setProcessChannelMode(QProcess::ForwardedErrorChannel);
     m_lockProcess->setReadChannel(QProcess::StandardOutput);
     auto finishedSignal = static_cast<void (QProcess::*)(int, QProcess::ExitStatus)>(&QProcess::finished);
     connect(m_lockProcess, finishedSignal, this,
@@ -256,7 +256,7 @@ void KSldApp::initialize()
     );
     connect(m_logind, &LogindIntegration::requestUnlock, this,
         [this]() {
-            if (lockState() == Locked) {
+            if (lockState() == Locked || lockState() == AcquiringLock) {
                 if (m_lockProcess->state() != QProcess::NotRunning) {
                     s_logindExit = true;
                     m_lockProcess->kill();
@@ -387,11 +387,7 @@ void KSldApp::lock(EstablishLock establishLock)
 
     // start unlock screen process
     startLockProcess(establishLock);
-}
-
-KActionCollection *KSldApp::actionCollection()
-{
-    return m_actionCollection;
+    emit lockStateChanged();
 }
 
 /*
@@ -540,18 +536,12 @@ void KSldApp::doUnlock()
                          KNotification::CloseOnTimeout,
                          QStringLiteral("ksmserver"));
     emit unlocked();
+    emit lockStateChanged();
 }
 
 bool KSldApp::isFdoPowerInhibited() const
 {
-    QDBusMessage msg = QDBusMessage::createMethodCall(QStringLiteral("org.kde.Solid.PowerManagement.PolicyAgent"),
-                                                      QStringLiteral("/org/kde/Solid/PowerManagement/PolicyAgent"),
-                                                      QStringLiteral("org.kde.Solid.PowerManagement.PolicyAgent"),
-                                                      QStringLiteral("HasInhibition"));
-    msg << (uint) 5; // PowerDevil::PolicyAgent::RequiredPolicy::ChangeScreenSettings | PowerDevil::PolicyAgent::RequiredPolicy::InterruptSession
-    QDBusReply<bool> reply = QDBusConnection::sessionBus().asyncCall(msg);
-
-    return reply.isValid() && reply.value();
+    return m_powerManagementInhibition->isInhibited();
 }
 
 void KSldApp::startLockProcess(EstablishLock establishLock)
@@ -573,7 +563,7 @@ void KSldApp::startLockProcess(EstablishLock establishLock)
         emit greeterClientConnectionChanged();
         int socket = dup(sx[1]);
         if (socket >= 0) {
-            QProcessEnvironment env = QProcessEnvironment::systemEnvironment();
+            QProcessEnvironment env = m_greeterEnv;
             env.insert("WAYLAND_SOCKET", QByteArray::number(socket));
             m_lockProcess->setProcessEnvironment(env);
         }
@@ -608,10 +598,10 @@ void KSldApp::showLockWindow()
 {
     if (!m_lockWindow) {
         if (m_isX11) {
-            m_lockWindow = new X11Locker();
+            m_lockWindow = new X11Locker(this);
         }
         if (m_isWayland) {
-            m_lockWindow = new WaylandLocker();
+            m_lockWindow = new WaylandLocker(m_waylandDisplay, this);
         }
         if (!m_lockWindow) {
             return;
@@ -700,9 +690,31 @@ void KSldApp::setWaylandDisplay(KWayland::Server::Display *display)
 
 void KSldApp::lockScreenShown()
 {
+    if (m_lockState == Locked) {
+        return;
+    }
     m_lockState = Locked;
     m_lockedTimer.restart();
     emit locked();
+    emit lockStateChanged();
+}
+
+void KSldApp::setGreeterEnvironment(const QProcessEnvironment &env)
+{
+    m_greeterEnv = env;
+    if (m_isWayland) {
+        m_greeterEnv.insert(QStringLiteral("QT_QPA_PLATFORM"), QStringLiteral("wayland"));
+    }
+}
+
+bool KSldApp::event(QEvent *event)
+{
+    if (m_globalAccel && event->type() == QEvent::KeyPress) {
+        if (m_globalAccel->keyEvent(static_cast<QKeyEvent*>(event))) {
+            event->setAccepted(true);
+        }
+    }
+    return false;
 }
 
 } // namespace
